@@ -627,7 +627,12 @@ isNS = Boolean(navigator.productSub)
 
 isIE = (ver.indexOf("MSIE 5") != -1 || ver.indexOf("MSIE 6") != -1) ? 1 : 0;
 isIE55 = (ver.indexOf("MSIE 5.5") != -1) ? 1 : 0;
-isKonqueror
 isOTHER = (!isNS && !isIE) ? 1 : 0;
 
-if(isOTHER) alert("Sorry your browser doesn't support the features of vcXMLRPC");
+// if isKonqueror and version> 3.2 then OK
+try{ x = new XMLHttpRequest()
+       isNS=true;
+}
+catch(e){
+ alert("Sorry your browser doesn't support the features of vcXMLRPC");
+		}
