@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: data.py 7686 2005-07-05 18:23:28Z reinhard $
+# $Id: data.py 7995 2005-09-26 08:32:05Z johannes $
 
 import copy
 import weakref
@@ -530,7 +530,8 @@ class _cache:
       self.__old.clear ()
       self.__new.clear ()
       self.__state.clear ()
-
+      self.inserted.clear ()
+      self.deleted.clear ()
 
 
   # ---------------------------------------------------------------------------
@@ -747,10 +748,7 @@ class connection:
 
     self.__uuidType = gConfig ('uuidtype').lower ()
 
-    # Keep a weak reference to the backend connection and make sure we're
-    # logged in
-    self.__backend = weakref.ref (connections.getConnection (database))
-    connections.loginToConnection (self.__backend ())
+    self.__backend = connections.getConnection (database, True)
 
 
   # ---------------------------------------------------------------------------
@@ -895,7 +893,7 @@ class connection:
 
     # otherwise requery the current record for all uncached fields
     else:
-      new = self.__backend ().requery (table, {u'gnue_id': row}, uncachedFields)
+      new = self.__backend.requery (table, {u'gnue_id': row}, uncachedFields)
       r = record (self.__cache, self.__connections, self.__database, table, row)
       r._fill (None, uncachedFields, new)
 
@@ -934,37 +932,62 @@ class connection:
 
     tables = self.__cache.dirtyTables ()
 
-    backend = self.__backend ()
+    # We distinguish 2 cases here:
+    # A: Backend knows transactions and needs a rollback when an exception has
+    # happened: we send all changes to the backend and leave the cache
+    # unchanged until the commit has succeeded, so we can restore to the state
+    # before in case of an exception. With this case, we will never have any
+    # uncommitted changes hanging around in the backend.
+    # B: Backend does not know transactions or does not need a rollback when an
+    # exception has happened: we clean up each single change from the cache as
+    # soon as we have sent it to the backend, so in case of an exception we can
+    # later continue where we've stopped. With this case, some changes at the
+    # backend might remain uncommitted until the commit is tried again without
+    # exception.
 
-    # first perform all inserts
-    if self.__cache.inserted:
-      for (table, row) in self.__orderInserts ():
-        fields = tables [table] [row]
+    try:
+      # first perform all inserts
+      if self.__cache.inserted:
+        for (table, row) in self.__orderInserts ():
+          fields = tables [table] [row]
+          self.__backend.insert (table, fields)
+          if not self.__backend._need_rollback_after_exception_:
+            self.__cache.makeClean (table, row)
 
-        backend.insert (table, fields)
-        self.__cache.makeClean (table, row)
+      # second perform all updates
+      for (table, rows) in tables.items ():
+        for (row, fields) in rows.items ():
+          if self.__cache.state (table, row) == 'changed':
+            self.__backend.update (table, {'gnue_id': row}, fields)
+            if not self.__backend._need_rollback_after_exception_:
+              self.__cache.makeClean (table, row)
 
+      # perform all deletes
+      if len (self.__cache.deleted):
+        for (table, row) in self.__orderDeletes ():
+          self.__backend.delete (table, {'gnue_id': row})
+          if not self.__backend._need_rollback_after_exception_:
+            self.__cache.remove (table, row)
 
-    # second perform all updates
-    for (table, rows) in tables.items ():
-      for (row, fields) in rows.items ():
-        if self.__cache.state (table, row) == 'changed':
-          backend.update (table, {'gnue_id': row}, fields)
-          self.__cache.makeClean (table, row)
+      # Commit the whole transaction
+      self.__backend.commit ()
 
+      if self.__backend._need_rollback_after_exception_:
+        for (table, rows) in tables.items ():
+          for row in rows.keys ():
+            state = self.__cache.state (table, row)
+            if state in ['inserted', 'changed']:
+              self.__cache.makeClean (table, row)
+            elif state == 'deleted':
+              self.__cache.remove (table, row)
 
-    # perform all deletes
-    if len (self.__cache.deleted):
-      for (table, row) in self.__orderDeletes ():
-        backend.delete (table, {'gnue_id': row})
-        self.__cache.remove (table, row)
+    except:
+      if self.__backend._need_rollback_after_exception_:
+        self.__backend.rollback ()
+      raise
 
-
-    # Commit the whole transaction
-    self.__connections.commitAll ()
-
-    # The transaction has ended. Changes from other transactions could become
-    # valid in this moment, so we have to clear the cache.
+    # Clear any old stuff from cache, but keep initialized (and not
+    # commitbable) records in there.
     self.__cache.clear (True)
 
 
@@ -983,7 +1006,7 @@ class connection:
     records = [(table, row) for (table, row) in self.__cache.inserted.values ()\
                              if self.__cache.state (table, row) == 'inserted']
     result = self.__orderByDependency (records)
-    gDebug (1, "Ordered inserts: %s" % result)
+    assert gDebug (1, "Ordered inserts: %s" % result)
 
     return result
 
@@ -1025,8 +1048,8 @@ class connection:
     data   = {}
     fishes = {}
 
-    gDebug (1, "Constraints: %s" % self.__constraints)
-    gDebug (1, "Unordered  : %s" % records)
+    assert gDebug (1, "Constraints: %s" % self.__constraints)
+    assert gDebug (1, "Unordered  : %s" % records)
 
     # First create a dictionary with all tables scheduled for processing, and
     # another dictionary of all records grouped by their tablename
@@ -1172,7 +1195,7 @@ class connection:
     # written anything yet, we have to tell the database that a new transaction
     # starts now, so that commits from other sessions become valid now for us
     # too.
-    self.__connections.rollbackAll ()
+    self.__backend.rollback ()
 
     # The transaction has ended. Changes from other transactions could become
     # valid in this moment, so we have to clear the whole cache.
@@ -1188,7 +1211,7 @@ class connection:
     Close the connection to the database backend.
     """
 
-    self.__connections.closeAll ()
+    self.__backend.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -1850,11 +1873,9 @@ class record:
     # If we have no backend reference until now, create a weak reference to the
     # connection instance and make sure we're logged in.
     if self.__backend is None:
-      cons = self.__connections
-      self.__backend = weakref.ref (cons.getConnection (self.__database))
-      cons.loginToConnection (self.__backend ())
+      self.__backend = self.__connections.getConnection (self.__database, True)
 
-    new = self.__backend ().requery (self.__table, {u'gnue_id': self.__row},
+    new = self.__backend.requery (self.__table, {u'gnue_id': self.__row},
                                      [field])
     if new:
       value = new.get (field)
