@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: readgcd.py 7829 2005-08-12 14:51:18Z reinhard $
+# $Id: readgcd.py 7915 2005-09-19 09:43:52Z johannes $
 
 import sys
 import os
@@ -313,8 +313,14 @@ class gcdReader:
 
     self.__integrityCheck ()
 
-    self.executeAndGenerateCode ()
-    self.updateRepository ()
+    self.__backend = self._connections.getConnection (self._database, True)
+
+    try:
+      self.executeAndGenerateCode ()
+      self.updateRepository ()
+
+    finally:
+      self.__backend.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -329,11 +335,8 @@ class gcdReader:
     given output file (if requested by options).
     """
 
-    connection = self._connections.getConnection (self._database)
-    self._connections.loginToConnection (connection)
-
     print _("Updating schema ...")
-    code = connection.writeSchema (self.schema, self._fileonly)
+    code = self.__backend.writeSchema (self.schema, self._fileonly)
 
     if self._dump is not None:
       dest = open (self._dump, 'w')
@@ -362,7 +365,7 @@ class gcdReader:
     commit |= self.__updateParameter ()
 
     if commit:
-      self._connections.commitAll ()
+      self.__backend.commit ()
 
 
   # ---------------------------------------------------------------------------
