@@ -19,15 +19,13 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasGsdGen.py 7641 2005-06-22 13:58:32Z johannes $
+# $Id: geasGsdGen.py 7898 2005-09-12 14:31:40Z jamest $
 
 import sys
 import copy
+import datetime
 import mx.DateTime
 
-if sys.version_info [:2] >= (2, 3):
-  import datetime
-
 from gnue.common.apps import i18n, errors
 from gnue.common.apps.i18n import translate as _        # for epydoc
 from gnue.common.apps.GClientApp import *
@@ -104,7 +102,7 @@ class geasGsdGen (GClientApp):
     cparser = self.configurationManager._loadedConfigs ['gnue.conf']
 
     if self.OPTIONS ["database"] is not None:
-      gDebug (1, "DEPRECIATION WARNING: use of 'database' option is "
+      assert gDebug (1, "DEPRECIATION WARNING: use of 'database' option is "
                  "depreciated. Use '--connection' instead.")
       cparser.set ('appserver', 'connection', self.OPTIONS ["database"])
 
@@ -356,8 +354,7 @@ class geasGsdGen (GClientApp):
       if isinstance (native, mx.DateTime.DateTimeType):
         return native.date
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (native, datetime.date):
+      elif isinstance (native, datetime.date):
         return native.isoformat ()
 
       else:
@@ -371,12 +368,10 @@ class geasGsdGen (GClientApp):
       elif isinstance (native, mx.DateTime.DateTimeDeltaType):
         return str (native)
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (native, datetime.time):
+      elif isinstance (native, datetime.time):
         return native.isoformat ()
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (native, datetime.timedelta):
+      elif isinstance (native, datetime.timedelta):
         return "%s" % native
 
       else:
@@ -387,8 +382,7 @@ class geasGsdGen (GClientApp):
       if isinstance (native, mx.DateTime.DateTimeType):
         return str (native)
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (native, datetime.datetime):
+      elif isinstance (native, datetime.datetime):
         return native.isoformat ()
 
       else:
@@ -443,7 +437,7 @@ class geasGsdGen (GClientApp):
 
         result [parts [0]][field] = value
 
-    gDebug (1, "Filter-Params: %s" % result)
+    assert gDebug (1, "Filter-Params: %s" % result)
     return result
 
 
@@ -515,7 +509,7 @@ class geasGsdGen (GClientApp):
 
         result [filterId] = match
 
-    gDebug (1, "Filters: %s" % result)
+    assert gDebug (1, "Filters: %s" % result)
     return result
 
       
@@ -723,7 +717,7 @@ class geasGsdGen (GClientApp):
       if len (cond):
         result [className] = cond
 
-    gDebug (1, "Condition-Dict: %s" % result)
+    assert gDebug (1, "Condition-Dict: %s" % result)
     return result
 
 
