@@ -19,19 +19,17 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasInstance.py 7795 2005-08-07 18:44:59Z reinhard $
+# $Id: geasInstance.py 7901 2005-09-14 17:25:29Z johannes $
 
 import sys
+import datetime
 import mx.DateTime
-import mx.DateTime.ISO
-
-if sys.version_info [:2] >= (2, 3):
-  import datetime
 
 from gnue import appserver
 from gnue.appserver.language import Object, Session
 from gnue.common.logic.language import getLanguageAdapter
 from gnue.common.apps import errors
+from gnue.common.utils import GDateTime
 
 # =============================================================================
 # Exceptions
@@ -140,65 +138,61 @@ class geasInstance:
         raise exception (propertydef.fullName, value)
       
     elif propertydef.dbType == "date":
-      # Date property: Must be an mx.DateTime or at least parseable as such
-      if isinstance (value, unicode):
-        value = mx.DateTime.ISO.ParseDateTime (value.encode ('utf-8') + \
-                                               ' 00:00:00')
-      elif isinstance (value, str):
-        value = mx.DateTime.ISO.ParseDateTime (value + ' 00:00:00')
+      # Date property: Must be a datetime.date or at least parseable as such
+      if isinstance (value, basestring):
+        value = GDateTime.parseISO (value)
 
-      elif isinstance (value, datetime.datetime):
-        value = mx.DateTime.ISO.ParseDateTime ((value.date()).isoformat () \
-            + ' 00:00:00')
+      if isinstance (value, datetime.datetime):
+        return value.date ()
 
       elif isinstance (value, datetime.date):
-        value = mx.DateTime.ISO.ParseDateTime (value.isoformat () + ' 00:00:00')
+        return value
+
+      elif isinstance (value, mx.DateTime.DateTimeType):
+        return datetime.date (value.year, value.month, value.day)
 
-      if isinstance (value, mx.DateTime.DateTimeType):
-        return mx.DateTime.DateTime (value.year, value.month, value.day,
-                                     0, 0, 0)
       else:
         raise exception (propertydef.fullName, value)
 
     elif propertydef.dbType == "time":
-      # Time property: Must be an mx.DateTime or at least parseable as such
-      if isinstance (value, unicode):
-        value = mx.DateTime.ISO.ParseDateTime ('0001-01-01 ' + \
-                                               value.encode ('utf-8'))
-      elif isinstance (value, str):
-        value = mx.DateTime.ISO.ParseDateTime ('0001-01-01 ' + value)
+      # Time property: Must be a datetime.time or at least parseable as such
+      if isinstance (value, basestring):
+        value = GDateTime.parseISO (value)
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (value, datetime.time):
-        value = mx.DateTime.ISO.ParseDateTime ('0001-01-01 ' + \
-                                               value.isoformat ())
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (value, datetime.timedelta):
-        value = datetime.datetime (1, 1, 1) + value
-        value = mx.DateTime.ISO.ParseDateTime (value.isoformat ())
+      if isinstance (value, datetime.datetime):
+        return value.time ()
 
-      if isinstance (value, mx.DateTime.DateTimeType) or \
+      elif isinstance (value, datetime.time):
+        return value
+
+      elif isinstance (value, datetime.timedelta):
+        return (datetime.datetime (1, 1, 1) + value).time ()
+
+      elif isinstance (value, mx.DateTime.DateTimeType) or \
          isinstance (value, mx.DateTime.DateTimeDeltaType):
-        return mx.DateTime.DateTime (1, 1, 1,
-                                     value.hour, value.minute, value.second)
+        return datetime.time (value.hour, value.minute, int (value.second),
+            int ((value.second - int (value.second)) * 1000000))
+
       else:
         raise exception (propertydef.fullName, value)
 
     elif propertydef.dbType == "datetime":
-      # Date/Time property: Must be an mx.DateTime or at least parseable as
-      # such
-      if isinstance (value, unicode):
-        return mx.DateTime.ISO.ParseDateTime (value.encode ('utf-8'))
+      # Date/Time property: Must be a datetime.datetime or at least parseable
+      # as such
+      if isinstance (value, basestring):
+        value = GDateTime.parseISO (value)
 
-      elif isinstance (value, str):
-        return mx.DateTime.ISO.ParseDateTime (value)
-
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (value, datetime.datetime):
-        return mx.DateTime.ISO.ParseDateTime (value.isoformat ())
-
-      if isinstance (value, mx.DateTime.DateTimeType):
+      if isinstance (value, datetime.datetime):
         return value
+
+      elif isinstance (value, datetime.date):
+        return datetime.datetime (value.year, value.month, value.day)
+
+      elif isinstance (value, mx.DateTime.DateTimeType):
+        return datetime.datetime (value.year, value.month, value.day,
+            value.hour, value.minute, int (value.second),
+            int ((value.second - int (value.second)) * 1000000))
+
       else:
         raise exception (propertydef.fullName, value)
 
@@ -252,7 +246,20 @@ class geasInstance:
     else:
       value = record.getField (propertydef.column)
 
-    return self.__convert (value, propertydef, DbValueError)
+    value = self.__convert (value, propertydef, DbValueError)
+
+    # FIXME: For compatibility, return mx.DateTime instead of datetime objects.
+    # Remove this for 0.5
+    if isinstance (value, datetime.datetime):
+      return mx.DateTime.DateTime (value.year, value.month, value.day,
+          value.hour, value.minute, value.second + value.microsecond / 1000000)
+    elif isinstance (value, datetime.date):
+      return mx.DateTime.DateTime (value.year, value.month, value.day, 0, 0, 0)
+    elif isinstance (value, datetime.time):
+      return mx.DateTime.DateTimeDelta (0, value.hour, value.minute,
+          value.second + value.microsecond / 1000000)
+    else:
+      return value
 
   # ---------------------------------------------------------------------------
   # Get the values of a list of properties
@@ -422,7 +429,7 @@ class geasInstance:
     (datefield, userfield) = [('gnue_modifydate', 'gnue_modifyuser'),
                               ('gnue_createdate', 'gnue_createuser')][creation]
     if self.has_key (datefield):
-      self.__putValue (datefield, mx.DateTime.now (), False)
+      self.__putValue (datefield, datetime.datetime.now (), False)
 
     if self.has_key (userfield) and self.__session.user is not None:
       self.__putValue (userfield, self.__session.user, False)
