@@ -19,9 +19,9 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: classdef.py 7194 2005-03-13 10:41:15Z johannes $
+# $Id: classdef.py 7872 2005-08-20 11:41:58Z reinhard $
 
-import mx
+import datetime
 
 from gnue.appserver import repository
 from gnue.common.apps import errors
@@ -404,7 +404,8 @@ class Property:
       self.typecast    = 'date'
       self.inputmask   = masks [typeId]
       self.displaymask = masks [typeId]
-      self.fieldLength = len (mx.DateTime.now ().strftime (self.displaymask))
+      self.fieldLength = \
+          len ((datetime.datetime.now ()).strftime (self.displaymask))
 
     elif propDef.gnue_type == 'number':
       self.typecast = 'number'
