@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: form.py 7641 2005-06-22 13:58:32Z johannes $
+# $Id: form.py 7872 2005-08-20 11:41:58Z reinhard $
 
 import classdef
 import layout
@@ -211,23 +211,12 @@ class FormGenerator:
       for item in self.__classDef.properties:
         width.append (max (len (item.label), item.widgetWidth ()))
 
-      if self.__sum (width) + len (width) - 1 <= self.__maxWidth:
+      if sum (width) + len (width) - 1 <= self.__maxWidth:
         result = layout.Tabular
 
     return result (self.__classDef, self.__maxWidth, self.__maxHeight)
 
 
-  # ---------------------------------------------------------------------------
-  # Create the sum of a sequence of numbers
-  # ---------------------------------------------------------------------------
-
-  def __sum (self, *args):
-    if sys.version_info [0:2] >= (2, 3):
-      return sum (*args)
-    else:
-      return reduce (operator.add, *args)
-
-
   # ---------------------------------------------------------------------------
   # Get an XML representation of all blocks and their fields
   # ---------------------------------------------------------------------------
