@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: layout.py 7463 2005-04-22 16:54:38Z reinhard $
+# $Id: layout.py 7872 2005-08-20 11:41:58Z reinhard $
 
 import math
 import operator
@@ -342,7 +342,7 @@ class Vertical (Manager):
     result   = 0
 
     for (page, properties) in self._classdef.virtualPages:
-      need = self.__sum ([i.minHeight or self._MIN_HEIGHT for i in properties])
+      need = sum ([i.minHeight or self._MIN_HEIGHT for i in properties])
 
       # if the needed space exceeds the longest page possible, or a page
       # contains stretchable items, use the maximum available space
@@ -354,18 +354,6 @@ class Vertical (Manager):
     return result
 
 
-  # ---------------------------------------------------------------------------
-  # Create the sum of a sequence of numbers
-  # ---------------------------------------------------------------------------
-
-  def __sum (self, *args):
-
-    if sys.version_info [0:2] >= (2, 3):
-      return sum (*args)
-    else:
-      return reduce (operator.add, *args)
-
-
   # ---------------------------------------------------------------------------
   # Get the next set of properties for a given page
   # ---------------------------------------------------------------------------
