@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: readgld.py 7829 2005-08-12 14:51:18Z reinhard $
+# $Id: readgld.py 7915 2005-09-19 09:43:52Z johannes $
 
 import sys
 import os
@@ -247,7 +247,7 @@ class gldReader:
       self.__currentFile, stream = self._files.popitem ()
 
       try:
-        gDebug (1, "Loading gld file '%s'" % self.__currentFile)
+        assert gDebug (1, "Loading gld file '%s'" % self.__currentFile)
 
         self.__currentModule = None
         self.__currentClass  = None
@@ -271,8 +271,14 @@ class gldReader:
     self.__replaceReferences ()
 
     # and finally update the repository
-    self.__updateLabels ()
-    self.__updateMessages ()
+    self.__backend = self._connections.getConnection (self._database, True)
+
+    try:
+      self.__updateLabels ()
+      self.__updateMessages ()
+
+    finally:
+      self.__backend.close ()
 
 
 
@@ -631,13 +637,13 @@ class gldReader:
       stat = self.__processResultSet (rSet, self.labels, cond)
 
       if stat [0] + stat [1]:
-        self._connections.commitAll ()
+        self.__backend.commit ()
 
       msg = u_("  Labels  : %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
                "unchanged.") \
             % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
 
-      gDebug (1, msg)
+      assert gDebug (1, msg)
       print o(msg)
 
     finally:
@@ -664,13 +670,13 @@ class gldReader:
       stat = self.__processResultSet (rSet, self.messages, cond)
 
       if stat [0] + stat [1]:
-        self._connections.commitAll ()
+        self.__backend.commit ()
 
       msg = u_("  Messages: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
                   "unchanged.") \
             % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
 
-      gDebug (1, msg)
+      assert gDebug (1, msg)
       print o(msg)
 
     finally:
@@ -785,7 +791,7 @@ class gldReader:
 
     if doPost and not deferPost:
       resultSet.post ()
-      resultSet.requery ()
+      resultSet.requery (False)
 
     return doPost
 
