@@ -19,9 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: ObjectList.py 7070 2005-02-28 13:35:22Z reinhard $
-
-import types
+# $Id: ObjectList.py 7641 2005-06-22 13:58:32Z johannes $
 
 from Object import Object
 from gnue.common.datasources import GConditions
@@ -78,7 +76,7 @@ class ObjectList:
   # -------------------------------------------------------------------------
   def __getitem__ (self, index):
     try:
-      if type (index) == types.SliceType:
+      if isinstance (index, slice):
         if index.start < 0:
           self.__fillup ()
 
@@ -157,30 +155,20 @@ class ObjectList:
     if conditions is None or not len (conditions):
       return None
 
-    if isinstance (conditions, types.ListType):
-      cTree = GConditions.buildTreeFromList (conditions)
+    cTree = GConditions.buildCondition (conditions)
 
-    elif isinstance (conditions, types.DictType):
-      cTree = GConditions.buildConditionFromDict (conditions)
+    # Replace all constant values represented by an Object instance with the
+    # apropriate gnue_id
+    for item in cTree.findChildrenOfType ('GCCConst', True, True):
+      if isinstance (item.value, Object):
+        item.value = item.value.gnue_id
 
-    else:
-      cTree = conditions
+    # Make sure all field names are fully qualified
+    for item in cTree.findChildrenOfType ('GCCField', True, True):
+      item.name = self.__session.qualifyMultiple (item.name)
 
-    try:
-      # Replace all constant values represented by an Object instance with the
-      # apropriate gnue_id
-      for item in cTree.findChildrenOfType ('GCCConst', True, True):
-        if isinstance (item.value, Object):
-          item.value = item.value.gnue_id
+    return cTree.prefixNotation ()
 
-      # Make sure all field names are fully qualified
-      for item in cTree.findChildrenOfType ('GCCField', True, True):
-        item.name = self.__session.qualifyMultiple (item.name)
-
-      return cTree.prefixNotation ()
-
-    finally:
-      cTree.breakReferences ()
 
 
   # ---------------------------------------------------------------------------
@@ -214,13 +202,13 @@ class ObjectList:
     result = []
 
     for item in sorting:
-      if isinstance (item, types.DictType):
+      if isinstance (item, dict):
         item ['name'] = self.__session.qualifyMultiple (item ['name'])
 
-      elif isinstance (item, types.ListType):
+      elif isinstance (item, list):
         item [0] = self.__session.qualifyMultiple (item [0])
 
-      elif isinstance (item, types.TupleType):
+      elif isinstance (item, tuple):
         data = list (item)
         data [0] = self.__session.qualifyMultiple (data [0])
         item = tuple (data)
