@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: Procedure.py 7104 2005-03-07 16:49:25Z johannes $
+# $Id: Procedure.py 7939 2005-09-21 11:17:08Z johannes $
 
 # =============================================================================
 # Implement a callable environment for the language interface 
@@ -40,11 +40,12 @@ class Procedure:
   # Delegate calls of this object to the session manager
   # ---------------------------------------------------------------------------
   def __call__ (self, **params):
-    gEnter (6)
+    assert gEnter (6)
 
     sm  = self.__session.getSessionManager ()
     sid = self.__session.getSessionId ()
 
     result = sm.call (sid, self.__class, [self.__objectId], self.method,
                     params) [0]
-    return gLeave (6, result)
+    assert gLeave (6, result)
+    return result
