@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: repository.py 7360 2005-04-13 14:00:02Z johannes $
+# $Id: repository.py 7999 2005-09-27 02:45:49Z reinhard $
 
 from gnue.common.apps import errors
 from gnue.common.utils.CaselessDict import CaselessDict
@@ -64,6 +64,24 @@ class ParameterNotFoundError (errors.ApplicationError):
           % {"procedure": procedure, "parameter": parameter}
     errors.ApplicationError.__init__ (self, msg)
 
+class LabelNotFoundError (errors.ApplicationError):
+  def __init__ (self, name, prname):
+    msg = u_("Property '%(prname)s' has no label '%(name)s'") \
+          % {"prname": prname, "name": name}
+    errors.ApplicationError.__init__ (self, msg)
+
+class ProcLabelNotFoundError (errors.ApplicationError):
+  def __init__ (self, name, prname):
+    msg = u_("Procedure '%(prname)s' has no label '%(name)s'") \
+          % {"prname": prname, "name": name}
+    errors.ApplicationError.__init__ (self, msg)
+
+class ClassLabelNotFoundError (errors.ApplicationError):
+  def __init__ (self, language, classname):
+    msg = u_("Class '%(class)s' has no label for language '%(language)s'") \
+        % {'class': classname, 'language': language}
+    errors.ApplicationError.__init__ (self, msg)
+
 class MissingFilterClassError (errors.ApplicationError):
   def __init__ (self, classname, filterid):
     msg = u_("Filter '%(filter)s' defined in class '%(classname)s' not found "
@@ -94,6 +112,12 @@ class InvalidNameError (errors.ApplicationError):
     msg = u_("'%s' is not a valid, fully qualified identifier") % name
     errors.ApplicationError.__init__ (self, name)
 
+class CircularFilterError (errors.ApplicationError):
+  def __init__ (self):
+    msg = u_("The filters have circular references")
+    errors.ApplicationError.__init__ (self, msg)
+
+
 # =============================================================================
 # This class implements the class repository
 # =============================================================================
@@ -116,18 +140,20 @@ class Repository:
   # Create a new repository instance
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session):
+  def __init__ (self, connections):
     """
-    @param session: geasSession instance to use for data retrieval
+    Create a new repository instance.
+
+    @param connections: L{gnue.common.datasources.GConnections} instance
     """
 
-    self._session     = session
-    self._langSession = Session.InternalSession (session.sm)
-    self._connection  = data.connection (session.connections, session.database)
+    self._connections = connections
+    self._database    = gConfig ('connection')
 
     self.modules = None
     self.classes = None
     self.filters = None
+    self.filterOrder = None
 
 
   # ---------------------------------------------------------------------------
@@ -153,13 +179,13 @@ class Repository:
 
       # Wrap all modules
       for (key, value) in self.__rawModules.items ():
-        module = Module (self._langSession, value)
+        module = Module (value)
         modules [key] = module
         modules [module.fullName] = module
 
       # Wrap all classes and install them into the apropriate module
       for (key, value) in self.__rawClasses.items ():
-        aClass = Class (self._langSession, value, self.__rawModules)
+        aClass = Class (value, self.__rawModules)
         module = aClass.gnue_module
         modules [module]._installClass (aClass)
 
@@ -180,20 +206,29 @@ class Repository:
 
       # Wrap all properties and install them into the apropriate class
       for (key, value) in self.__rawProperties.items ():
-        aProperty = Property (self._langSession, value, self.__rawModules)
+        aProperty = Property (value, self.__rawModules)
         classes [aProperty.gnue_class]._installProperty (aProperty)
         properties [key] = aProperty
 
       # Wrap all procedures and link them into a lookup dictionary first
       for (key, value) in self.__rawProcedures.items ():
-        aProcedure = Procedure (self._langSession, value, self.__rawModules)
+        aProcedure = Procedure (value, self.__rawModules)
         procedures [key] = aProcedure
 
       # Now create all parameters and install them to their procedures.
       for (key, value) in self.__rawParameters.items ():
-        aParameter = Parameter (self._langSession, value)
+        aParameter = Parameter (value)
         procedures [aParameter.gnue_procedure]._installParameter (aParameter)
 
+      # Create the labels and install them into their properties or procedures
+      for (key, value) in self.__rawLabels.items ():
+        aLabel = Label (value)
+
+        if aLabel.gnue_property:
+          properties [aLabel.gnue_property]._installLabel (aLabel)
+        else:
+          procedures [aLabel.gnue_procedure]._installLabel (aLabel)
+
       # After having all parameters installed to their procedures, we can then
       # run their second level initialization. Here it'll be determined wether
       # a procedure is a calculated field or not. Finally the procedures will
@@ -210,7 +245,6 @@ class Repository:
       # calculated properties.
       map (verifyProcedure, procedures.values ())
 
-
       # To finish the build process, all references (unicode-strings) are
       # replaced by the corresponding wrapper instance. This way a
       # 'someClass.gnue_module.fullName' will work.
@@ -236,6 +270,8 @@ class Repository:
       if oldmodules is not None:
         self.release (oldmodules, oldclasses, oldfilters)
 
+    self.filterOrder = self.__orderFilters ()
+
 
   # ---------------------------------------------------------------------------
   # Release resources of the repository
@@ -263,6 +299,27 @@ class Repository:
     modules.clear ()
 
 
+  # ---------------------------------------------------------------------------
+  #
+  # ---------------------------------------------------------------------------
+
+  def getFilterDefinition (self, language):
+    """
+    """
+
+    result = []
+    dcon   = data.connection (self._connections, self._database)
+
+    try:
+      for fk in self.filterOrder:
+        result.append (self.filters [fk]._getFilterDefinition (language, dcon))
+
+    finally:
+      dcon.close ()
+
+    return result
+
+
   # ---------------------------------------------------------------------------
   # Load the raw data from all repository tables
   # ---------------------------------------------------------------------------
@@ -272,20 +329,31 @@ class Repository:
     This function reads all repository tables into dictionaries.
     """
 
-    self.__rawModules = self.__loadTable (u'gnue_module', [u'gnue_name'])
-    self.__rawClasses = self.__loadTable (u'gnue_class', \
+    self._connection = data.connection (self._connections, self._database)
+
+    try:
+      self.__rawModules = self.__loadTable (u'gnue_module', [u'gnue_name'])
+      self.__rawClasses = self.__loadTable (u'gnue_class', \
                                 [u'gnue_module', u'gnue_name', u'gnue_filter'])
-    self.__rawProperties = self.__loadTable (u'gnue_property', \
+      self.__rawProperties = self.__loadTable (u'gnue_property', \
                   [u'gnue_module', u'gnue_class', u'gnue_name', u'gnue_length',
-                   u'gnue_scale', u'gnue_nullable', u'gnue_type'])
-    self.__rawProcedures = self.__loadTable (u'gnue_procedure', \
+                   u'gnue_scale', u'gnue_nullable', u'gnue_type',
+                   u'gnue_comment'])
+      self.__rawProcedures = self.__loadTable (u'gnue_procedure', \
                 [u'gnue_module', u'gnue_class', u'gnue_name', u'gnue_language',
                  u'gnue_length', u'gnue_nullable', u'gnue_scale', u'gnue_type',
-                 u'gnue_comment'])
+                 u'gnue_comment', u'gnue_code'])
 
-    self.__rawParameters = self.__loadTable (u'gnue_parameter', \
+      self.__rawParameters = self.__loadTable (u'gnue_parameter', \
                 [u'gnue_procedure', u'gnue_name', u'gnue_type', u'gnue_length',
-                 u'gnue_scale'])
+                 u'gnue_scale', u'gnue_comment'])
+
+      self.__rawLabels = self.__loadTable (u'gnue_label', \
+          [u'gnue_property', u'gnue_procedure', u'gnue_language', u'gnue_page',
+           u'gnue_label', u'gnue_position', u'gnue_search', u'gnue_info'])
+
+    finally:
+      self._connection.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -323,66 +391,83 @@ class Repository:
     return result
 
 
+  # ---------------------------------------------------------------------------
+  # Create an ordered list of filters
+  # ---------------------------------------------------------------------------
+
+  def __orderFilters (self):
+    
+    # Build a dependency tree of all filters
+    result  = []
+    filters = {}
+
+    for item in self.filters.values ():
+      rec = filters.setdefault (item.gnue_id, [])
+      if item.gnue_filter:
+        rec.append (item.gnue_filter.gnue_id)
+
+    # and transform that tree into a list starting with independent filters
+    while filters:
+      addition = []
+
+      for (fid, deps) in filters.items ():
+        # if a filter has no other dependencies, add it to the result and
+        # remove all other references to it
+        if not len (deps):
+          addition.append (fid)
+
+          for ref in filters.values ():
+            if fid in ref:
+              ref.remove (fid)
+
+          del filters [fid]
+
+      # Finally, if nothing was added to the result, but there are still
+      # filters left in the tree we have a circular reference problem
+      if not len (addition) and len (filters):
+        raise CircularFilterError
+
+      result.extend (addition)
+
+    return result
+
+
+
 # =============================================================================
 # Base class implementing a repository element
 # =============================================================================
 
 class BaseItem:
+  """
+  Base class for repository items. The optionally given dictionary will be
+  transferred into the instances __dict__ attribute.
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new instance of a repository item
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, classname, predefined = None):
+  def __init__ (self, classname, predefined = None):
     """
-    This function creates a new instance of a repository item. If a predefined
-    dictionary is given, all it's elements are copied into the instances'
-    __dict__. Access to other attributes will be delegated to the corresponding
-    language interface Object.
+    Create a new repository item instance.
 
-    @param session: language interface session
     @param classname: name of the class wrapped by this item
     @param predefined: dictionary with predefined values
     """
 
-    self._session  = session
-    self.__object  = None
     self.classname = classname
 
     if predefined is not None:
       self.__dict__.update (predefined)
 
 
-  # ---------------------------------------------------------------------------
-  # Access to a non-prefdefined attribute
-  # ---------------------------------------------------------------------------
-
-  def __getattr__ (self, attr):
-    """
-    This function delegates property access to the bound language interface
-    object and caches the result for following access.
-
-    @param attr: name of the attribute to retrieve
-    @return: value of the attribute
-    """
-
-    if self.__object is None:
-      self.__object = self._session._get (self.classname, self.gnue_id)
-
-    # Fetch the result of a given attribute and keep it cached for later reuse
-    result = getattr (self.__object, attr)
-    self.__dict__ [attr] = result
-
-    return result
-
-
   # ---------------------------------------------------------------------------
   # Dictionary emulation for wrapper instances
   # ---------------------------------------------------------------------------
 
   def __getitem__ (self, attr):
     """
-    This function emulates a dictionary access to the wrapper instance.
+    Emulate a dictionary access to the wrapper instance.
 
     @param attr: attribute to return
     @return: value of the attribute 'attr'
@@ -391,23 +476,81 @@ class BaseItem:
     return getattr (self, attr)
 
 
+  # ---------------------------------------------------------------------------
+  # Truth value testing
+  # ---------------------------------------------------------------------------
+
+  def __nonzero__ (self):
+
+    return True
+
+
+
+# =============================================================================
+# MixIn class for classes having gnue_labels
+# =============================================================================
+
+class LabelsMixIn:
+
+  # ---------------------------------------------------------------------------
+  # Add a label to the labels dictionary
+  # ---------------------------------------------------------------------------
+
+  def _installLabel (self, aLabel):
+    """
+    Add a label to the list of labels in the proper language.
+
+    @param aLabel: L{Label} to be added
+    """
+
+    self.labels [aLabel.gnue_language] = aLabel
+
+
+  # ---------------------------------------------------------------------------
+  # Get all labels usable within filter grouped by their language
+  # ---------------------------------------------------------------------------
+
+  def getFilterLabels (self, language):
+
+    usable = {}
+
+    for (lang, label) in self.labels.items ():
+      if label.order:
+        usable [lang] = label
+
+    for lang in getLanguages (language):
+      if lang.lower () in usable:
+        return usable [lang.lower ()]
+
+    return None
+
+
+
 # =============================================================================
 # This class implements a wrapper for module items of the repository
 # =============================================================================
 
 class Module (BaseItem):
+  """
+  A module object in the repository.
+
+  @ivar classes: a caseless dictionary of all classes of the module. Each class
+    will be listed in this dictionary by it's gnue_id as well as it's full name.
+  @ivar fullname: fully qualified name of the module
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new module item of the repository
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, predefined = None):
+  def __init__ (self, predefined = None):
     """
-    @param session: language interface session
+    Create a new module item.
+
     @param predefined: dictionary with predefined ('builtin') values
     """
 
-    BaseItem.__init__ (self, session, u'gnue_module', predefined)
+    BaseItem.__init__ (self, u'gnue_module', predefined)
 
     self.classes  = CaselessDict (ClassNotFoundError)
     self.fullName = self.gnue_name
@@ -419,8 +562,9 @@ class Module (BaseItem):
 
   def _installClass (self, aClass):
     """
-    This function adds the given class to the modules' list of classes.
-    @param aClass: Class instance to be added
+    Add a class element to the class dictionary.
+
+    @param aClass: L{Class} instance to be added
     """
 
     self.classes [aClass.gnue_id]  = aClass
@@ -433,15 +577,14 @@ class Module (BaseItem):
 
   def updateLinks (self, modules, classes, unlink = False):
     """
-    This class updates links of a module item by updating the links of all it's
-    classes.
+    Update links of a module item by updating the links of all it's classes.
 
     @param modules: dictionary with all modules available
     @param classes: dictionary with all classes available
     @param unlink: if set to True, references will be cleared
     """
 
-    for (key, item) in self.classes.items ():
+    for item in self.classes.values ():
       item.updateLinks (modules, classes, unlink)
 
 
@@ -451,19 +594,43 @@ class Module (BaseItem):
 # =============================================================================
 
 class Class (BaseItem):
+  """
+  A class object in the repository.
+
+  @ivar fullName: fully qualified name (including the module name)
+  @ivar table: name of the underlying table in the backend database
+  @ivar properties: caseless dictionary with all properties of the class. Every
+    property is accessible either by it's gnue_id or it's full name. This
+    dictionary also contains I{calculated properties}
+  @ivar procedures: caseless dictionary with all procedures of the class. Every
+    procedure is accessible either by it's gnue_id or it's full name
+  @ivar masters: caseless dictionary of all the master classes
+    (master-/detail-relation). This dictionary contains the full name of the
+    master classes as keys, and a sequence of those properties referencing that
+    master class.
+  @ivar OnInit: list of L{Procedure} objects containing OnInit triggers of the
+    class
+  @ivar OnChange: list of L{Procedure} objects containing OnChange triggers of
+    the class
+  @ivar OnValidate: list of L{Procedure} objects containing OnValidate triggers
+    of the class
+  @ivar OnDelete: list of L{Procedure} objects containing OnDelete triggers of
+    the class
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new class item of the repository
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, predefined, moduleLookup):
+  def __init__ (self, predefined, moduleLookup):
     """
-    @param session: language interface session
+    Create a new class item.
+
     @param predefined: dictionary with predefined ('builtin') values
     @param moduleLookup: (raw) lookup-dictionary with all modules available
     """
 
-    BaseItem.__init__ (self, session, u'gnue_class', predefined)
+    BaseItem.__init__ (self, u'gnue_class', predefined)
 
     moduleName    = moduleLookup [self.gnue_module] ['gnue_name']
     self.fullName = createName (moduleName, self.gnue_name)
@@ -473,6 +640,9 @@ class Class (BaseItem):
     self.procedures = CaselessDict (ProcedureNotFoundError, self.fullName)
     self.masters    = CaselessDict (ClassNotFoundError)
 
+    self.labels       = CaselessDict (ClassLabelNotFoundError, self.fullName)
+    self.labels ['C'] = self.fullName
+
     self.OnInit     = []
     self.OnChange   = []
     self.OnValidate = []
@@ -485,7 +655,7 @@ class Class (BaseItem):
 
   def _installProperty (self, aProperty):
     """
-    This function installs a given property to the property dictionary
+    Add a property to the property dictionary
 
     @param aProperty: Property instance to be installed
     """
@@ -500,10 +670,9 @@ class Class (BaseItem):
 
   def _installProcedure (self, aProcedure):
     """
-    This function adds a procedure to the procedure dictionary. If the
-    procedure is a 'calculated property' it also adds it to the property
-    dictionary. If a procedure is a trigger it will be added to the apropriate
-    trigger sequence.
+    Add a procedure to the procedure dictionary. If the procedure is a
+    'calculated property', also add it to the property dictionary. If a
+    procedure is a trigger, add it to the apropriate trigger sequence.
 
     @param aProcedure: procedure to be installed
     """
@@ -551,7 +720,7 @@ class Class (BaseItem):
     Add a given class to the dictionary of master-classes.
 
     @param aProperty: name of the property holding the pointer to master-class
-    @param aMaster: class wrapper instance to be added
+    @param aMaster: L{Class} instance to be added
     """
 
     self.masters.setdefault (aMaster.fullName, []).append (aProperty)
@@ -563,7 +732,7 @@ class Class (BaseItem):
 
   def updateLinks (self, modules, classes, unlink = False):
     """
-    This class updates links of a class item by updating the links of all it's
+    Updates links of a L{Class} item by updating the links of all it's
     properties and procedures.
 
     @param modules: dictionary with all modules available
@@ -592,30 +761,135 @@ class Class (BaseItem):
       item.updateLinks (self, modules, unlink)
 
 
+  # ---------------------------------------------------------------------------
+  #
+  # ---------------------------------------------------------------------------
+
+  def getFilterLabels (self, language):
+    """
+    """
+
+    props = {}
+    for item in self.properties.values ():
+      if item.gnue_id in props:
+        continue
+
+      flabels = item.getFilterLabels (language)
+      if flabels:
+        props [item.gnue_id] = flabels
+
+    if not props:
+      labels = [Label ({u'gnue_id': None,
+                     u'gnue_property': self.properties [u'gnue_id'],
+                     u'gnue_procedure': None,
+                     u'gnue_label': u'gnue_id',
+                     u'gnue_language': u'C',
+                     u'gnue_search': 0,
+                     u'gnue_info': None,
+                     u'gnue_page': None,
+                     u'gnue_position': 0})]
+    else:
+      labels = props.values ()
+
+    order = [(l.order, l.gnue_label, l) for l in labels]
+    order.sort ()
+
+    return [label for (pos, text, label) in order]
+
+    
+  def getClassLabel (self, language):
+
+    for lang in getLanguages (language):
+      result = self.labels.get (lang)
+      if result:
+        return result
+
+    return self.fullName
+
+
+  def _getFilterDefinition (self, language, dataCon):
+    """
+    """
+
+    labels = self.getFilterLabels (language)
+
+    master = None
+    fields = [l.gnue_property.fullName for l in labels]
+    if self.gnue_filter:
+      master = self.gnue_filter.fullName
+      fields.append (master)
+
+    data      = {}
+    contents  = {None: (self.table, None, None, fields)}
+    resultSet = dataCon.query (contents, None, [{'name': u'gnue_id'}])
+
+    try:
+      rec = resultSet.nextRecord ()
+      while rec is not None:
+        if master:
+          group = data.setdefault (rec.getField (master), [])
+        else:
+          group = data.setdefault (None, [])
+
+        row = {}
+        for field in fields:
+          if field != master:
+            row [field] = rec.getField (field)
+
+        group.append (row)
+
+        rec = resultSet.nextRecord ()
+
+    finally:
+      resultSet.close ()
+
+
+    return ((self.fullName, self.getClassLabel (language)), \
+        [(l.gnue_label, l.search, l.gnue_property.fullName) for l in labels],
+        master, data)
+
 
 # =============================================================================
 # This class wraps a property item of the repository
 # =============================================================================
 
-class Property (BaseItem):
+class Property (LabelsMixIn, BaseItem):
+  """
+  A property object in the repository.
+
+  @ivar fullName: fully qualified name of the property
+  @ivar column: name of the column in the backend this property is bound to
+  @ivar isCalculated: if True, the property is a calculated field, based on a
+    L{Procedure}. In this case column is set to None
+  @ivar isReference: True, if the property is a reference to another L{Class}.
+  @ivar referencedClass: for a reference property this attribute contains the
+    L{Class} instance of the referenced class (= master class)
+  @ivar fullType: full datatype as used by gnue
+  @ivar dbFullType: full datatype as used at the backend
+  @ivar dbType: base datatype as used at the backend
+  @ivar dbLength: length used at the backend
+  @ivar dbScale: scale used at the backend
+  """
   
   # ---------------------------------------------------------------------------
   # Create a new property item
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, predefined, moduleLookup):
+  def __init__ (self, predefined, moduleLookup):
     """
-    @param session: language interface session
+    Create a new property item.
+
     @param predefined: dictionary with predefined ('builtin') values
     @param moduleLookup: (raw) lookup-dictionary with all modules available
     """
 
     updateTypeInfo (predefined)
-    BaseItem.__init__ (self, session, u'gnue_property', predefined)
+    BaseItem.__init__ (self, u'gnue_property', predefined)
 
     moduleName        = moduleLookup [self.gnue_module]['gnue_name']
     self.fullName     = createName (moduleName, self.gnue_name)
     self.column       = self.fullName
+    self.labels       = CaselessDict (LabelNotFoundError, self.fullName)
 
     self.isCalculated    = False
     self.isReference     = False
@@ -628,11 +902,11 @@ class Property (BaseItem):
 
   def updateLinks (self, aClass, modules, unlink = False):
     """
-    This function updates the links of a property item to it's module and
-    class. If unlink is set the references are cleared.
+    Update the links of a property item to it's L{Module} and L{Class}. If
+    unlink is set the references are cleared.
 
-    @param aClass: Class instance the property belongs to
-    @param modules: Module dictionary with all modules available
+    @param aClass: L{Class} instance the property belongs to
+    @param modules: module dictionary with all modules available
     @param unlink: this boolean Flag determines wether to establish links or to
         break them
     """
@@ -647,6 +921,9 @@ class Property (BaseItem):
       if not isinstance (self.gnue_module, Module):
         self.gnue_module = modules [self.gnue_module]
 
+    for item in self.labels.values ():
+      item.updateLinks (self, unlink)
+
 
   # ---------------------------------------------------------------------------
   # Finalize the definition of a property
@@ -654,8 +931,8 @@ class Property (BaseItem):
 
   def finalize (self, classes):
     """
-    This function verifies the type of the property and set's the reference
-    flags for rerference properties.
+    Verify the type of the property and set the reference flags for rerference
+    properties.
 
     @param classes: dictionary with all classes available
     """
@@ -670,13 +947,34 @@ class Property (BaseItem):
     if r is not None:
       classes [self.gnue_class].addMasterClass (self.fullName, r)
 
+    # a 'gnue_id' property defines the class labels (if any)
+    parent = classes [self.gnue_class]
+    if self.fullName == u'gnue_id':
+      for (lang, label) in self.labels.items ():
+        parent.labels [lang] = label.gnue_label
 
 
 # =============================================================================
 # This class wraps a calculated property (= special procedures)
 # =============================================================================
 
-class CalculatedProperty (BaseItem):
+class CalculatedProperty (LabelsMixIn, BaseItem):
+  """
+  A calculated property object in the repository. A calculated property is a
+  special kind of a L{Procedure}.
+
+  @ivar fullName: fully qualified name of the property
+  @ivar column: allways None
+  @ivar isReference: allways False
+  @ivar referencedClass: allways None
+  @ivar isCalculated: allways True
+  @ivar procedure: the L{Procedure} instance defining the property
+  @ivar fullType: full datatype as used by gnue
+  @ivar dbFullType: full datatype as used at the backend
+  @ivar dbType: base datatype as used at the backend
+  @ivar dbLength: length used at the backend
+  @ivar dbScale: scale used at the backend
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new calculated property
@@ -684,6 +982,8 @@ class CalculatedProperty (BaseItem):
 
   def __init__ (self, aProcedure):
     """
+    Create a new calculated property item.
+
     @param aProcedure: procedure to bind as calculated property
     """
 
@@ -698,7 +998,7 @@ class CalculatedProperty (BaseItem):
       'gnue_nullable': aProcedure.gnue_nullable}
     updateTypeInfo (predefined)
 
-    BaseItem.__init__ (self, aProcedure._session, u'gnue_procedure', predefined)
+    BaseItem.__init__ (self, u'gnue_procedure', predefined)
 
     self.fullName  = aProcedure.calcFullName
     self.column    = None
@@ -707,6 +1007,12 @@ class CalculatedProperty (BaseItem):
     self.referencedClass = None
     self.isCalculated    = True
     self.procedure       = aProcedure
+    self.labels          = CaselessDict (LabelNotFoundError, self.fullName)
+
+    # Make sure to have all labels of the procedure bound to the calculated
+    # field as well
+    for item in aProcedure.labels.values ():
+      self._installLabel (item)
 
 
   # ---------------------------------------------------------------------------
@@ -717,8 +1023,8 @@ class CalculatedProperty (BaseItem):
     """
     If in unlink mode this function clears the pointer to the bound procedure.
 
-    @param aClass: Class instance the property belongs to
-    @param modules: Module dictionary with all modules available
+    @param aClass: L{Class} instance the property belongs to
+    @param modules: module dictionary with all modules available
     @param unlink: this boolean Flag determines wether to establish links or to
         break them
     """
@@ -733,6 +1039,8 @@ class CalculatedProperty (BaseItem):
       if not isinstance (self.gnue_module, Module):
         self.gnue_module = modules [self.gnue_module]
 
+    for item in self.labels.values ():
+      item.updateLinks (self, unlink)
 
 
 
@@ -740,20 +1048,31 @@ class CalculatedProperty (BaseItem):
 # This class wraps a procedure item of the repository
 # =============================================================================
 
-class Procedure (BaseItem):
+class Procedure (LabelsMixIn, BaseItem):
+  """
+  A procedure object in the repository.
+
+  @ivar fullName: fully qualified name of the procedure
+  @ivar isCalculated: if True, this procedure matches all requirements of a
+    calculated property.
+  @ivar calcFullName: fully qualified name of the calculated property
+  @ivar calcName: name of the calculated property withouth any module name
+  @ivar parameters: caseless dictionary with all L{Parameter}s of the procedure
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new procedure wrapper
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, predefined, moduleLookup):
+  def __init__ (self, predefined, moduleLookup):
     """
-    @param session: language interface session
+    Create a new procedure item.
+
     @param predefined: dictionary with predefined ('builtin') values
     @param moduleLookup: (raw) lookup-dictionary with all modules available
     """
 
-    BaseItem.__init__ (self, session, u'gnue_procedure', predefined)
+    BaseItem.__init__ (self, u'gnue_procedure', predefined)
 
     moduleName    = moduleLookup [self.gnue_module]['gnue_name']
     self.fullName = createName (moduleName, self.gnue_name)
@@ -763,6 +1082,7 @@ class Procedure (BaseItem):
     self.calcName     = None
 
     self.parameters = CaselessDict (ParameterNotFoundError)
+    self.labels     = CaselessDict (ProcLabelNotFoundError, self.fullName)
 
 
   # ---------------------------------------------------------------------------
@@ -771,8 +1091,7 @@ class Procedure (BaseItem):
 
   def secondaryInit (self, moduleLookup):
     """
-    This function determines wether a procedure can be used as a calculated
-    field or not. If so the flag 'isCalcualted' will be set.
+    Check wether a procedure matches all requirements of a calculated property
 
     @param moduleLookup: lookup dictionary for modules
     """
@@ -792,8 +1111,7 @@ class Procedure (BaseItem):
 
   def _installParameter (self, aParameter):
     """
-    This function installs the given parameter to the procedure's parameter
-    dictionary.
+    Add a L{Parameter} to the procedure's parameter dictionary.
 
     @param aParameter: the parameter to be installed.
     """
@@ -808,11 +1126,11 @@ class Procedure (BaseItem):
 
   def updateLinks (self, aClass, modules, unlink = False):
     """
-    This function updates the links of a procedure item to it's module and
-    class. If unlink is set the references are cleared.
+    Update the links of a procedure item to it's module and class. If unlink is
+    set the references are cleared.
 
-    @param aClass: Class instance the procedure belongs to
-    @param modules: Module dictionary with all modules available
+    @param aClass: L{Class} instance the procedure belongs to
+    @param modules: module dictionary with all modules available
     @param unlink: this boolean Flag determines wether to establish links or to
         break them
     """
@@ -829,6 +1147,9 @@ class Procedure (BaseItem):
     for param in self.parameters.values ():
       param.updateLinks (self, unlink)
 
+    for item in self.labels.values ():
+      item.updateLinks (self, unlink)
+
 
 
 # =============================================================================
@@ -836,21 +1157,29 @@ class Procedure (BaseItem):
 # =============================================================================
 
 class Parameter (BaseItem):
+  """
+  A parameter object of the repository.
+
+  @ivar fullName: fully qualified name of the paramter
+  @ivar dbType: the type of the parameter in the backend database
+  """
 
   # ---------------------------------------------------------------------------
   # Create a new parameter wrapper instance
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, session, predefined):
+  def __init__ (self, predefined):
     """
-    @param session: language interface session
+    Create a new parameter item.
+
     @param predefined: dictionary with predefined ('builtin') values
     """
 
-    BaseItem.__init__ (self, session, u'gnue_parameter', predefined)
+    BaseItem.__init__ (self, u'gnue_parameter', predefined)
+
     self.fullName = self.gnue_name
 
-    if NONREF_TYPES.has_key (self.gnue_type):
+    if self.gnue_type in NONREF_TYPES:
       self.dbType = self.gnue_type
     else:
       self.dbType = REF_TYPE
@@ -875,6 +1204,58 @@ class Parameter (BaseItem):
       self.gnue_procedure = aProcedure
 
 
+# =============================================================================
+# Label class
+# =============================================================================
+
+class Label (BaseItem):
+  """
+  A label object in the repository.
+
+  @ivar order: the search order of a label which is either gnue_search or if
+    that is None gnue_info
+  @ivar search: True, if gnue_search is not None
+  """
+
+  # ---------------------------------------------------------------------------
+  # Create a new label item
+  # ---------------------------------------------------------------------------
+
+  def __init__ (self, predefined):
+    """
+    Create a new label item.
+
+    @param predefined: dictionary with predefined ('builtin') values
+    """
+
+    BaseItem.__init__ (self, u'gnue_label', predefined)
+
+    self.order  = self.gnue_search or self.gnue_info
+    self.search = self.gnue_search is not None
+
+
+  # ---------------------------------------------------------------------------
+  # Update links
+  # ---------------------------------------------------------------------------
+
+  def updateLinks (self, aParent, unlink = False):
+    """
+    Update the link to the owning property or procedure of a label
+
+    @param aParent: the owning property or procedure
+    @param unlink: if set to True the reference will be cleared
+    """
+
+    if unlink:
+      self.gnue_procedure = None
+      self.gnue_property  = None
+
+    else:
+      if self.gnue_property:
+        self.gnue_property  = aParent
+      else:
+        self.gnue_procedure = aParent
+
 
 # =============================================================================
 # Type checking support
@@ -898,13 +1279,16 @@ REF_SCALE    = 0
 
 def verifyBasetype (typename, length, scale):
   """
-  This function verifies a given typename with length and scale. If this
-  combination makes no sense a TypeFormatError will be raised. If typename
-  is no valid base type a TypeNameError will be raised.
+  Verify a given typename with length and scale. If this combination makes no
+  sense a TypeFormatError will be raised. If typename is no valid base type a
+  TypeNameError will be raised.
 
   @param typename: name of the datatype
   @param length: length of the datatype
   @param scale: scale of the datatype
+
+  @raises TypeFormatError: the combination of length and scale is not valid
+  @raises TypeNameError: typename is not a valid base type
   """
 
   if not BASE_TYPES.has_key (typename):
@@ -944,6 +1328,8 @@ def verifyType (typename, length, scale, classes):
 
   @return: If 'typename' is a reference type this function returns the class
       definition of this reference type, otherwise None
+
+  @raises TypeFormatError: the type, length and scale combination is not valid
   """
 
   if classes.has_key (typename):
@@ -1091,3 +1477,31 @@ def splitName (name):
     raise InvalidNameError, name
 
   return result
+
+
+# -----------------------------------------------------------------------------
+# Create a list of languages in descending order
+# -----------------------------------------------------------------------------
+
+def getLanguages (language):
+  """
+  Create an ordered list of languages to fetch labels for. The list starts with
+  the most specific language and contains at least 'C'. The language 'de_AT'
+  will result in a list ['de_AT', 'de', 'C'].
+
+  @param language: language (locale) to create a list for, e.g. 'de_AT'
+  @return: list of language-codes in descending order
+  """
+
+  result = []
+
+  if '_' in language:
+    result = [language.split ('_') [0]]
+
+  if not language in result:
+    result.insert (0, language)
+
+  if not 'C' in result:
+    result.append ('C')
+
+  return result
