@@ -19,9 +19,10 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: testRPC.py 6851 2005-01-03 20:59:28Z jcater $
+# $Id: testRPC.py 7889 2005-09-06 17:37:55Z johannes $
+
+from gnue.common.rpc import client
 
-from gnue.common.rpc import GComm
 import test
 
 # =============================================================================
@@ -32,7 +33,7 @@ class testRpcApp (test.testApp):
   # standart settings
   type   = "xmlrpc"
   params = {"host": "localhost",
-            "port": 8765,
+            "port": 4711,
             "transport": "http" }
 
   def setConnection(self,type,params):
@@ -45,12 +46,8 @@ class testRpcApp (test.testApp):
 
   def getSessionManager (self):
     print "Opening RPC connection ..."
-    server = GComm.attach (self.type, self.params)
+    return client.attach (self.type, self.params)
 
-    print "Setup an link to the session manager"
-    sm = server.request ("Session")
-
-    return sm
 
 # =============================================================================
 # Let it rock
